@@ -974,6 +974,3 @@ ta_export_signals = {
         },
     },
 }
-
-ta_export_allocations = {}
-ta_export = {PandasEnum.SIGNAL.value: ta_export_signals, PandasEnum.ALLOCATION.value: ta_export_allocations}
